@@ -1,14 +1,14 @@
-use bitbuffer::{BitRead, BitWrite, BitWriteStream, LittleEndian};
-
-use crate::{GameEventError, Result, Stream};
-
 pub use super::gameevent_gen::{GameEvent, GameEventType};
 use crate::demo::handle_utf8_error;
 use crate::demo::message::gameevent::GameEventTypeId;
+use crate::{GameEventError, Result, Stream};
+use bitbuffer::{BitRead, BitWrite, BitWriteStream, LittleEndian};
 use parse_display::Display;
+use serde::{Deserialize, Serialize};
 use std::cmp::Ordering;
 
-#[derive(Debug, Clone)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct GameEventDefinition {
     pub id: GameEventTypeId,
     pub event_type: GameEventType,
@@ -35,13 +35,15 @@ impl Ord for GameEventDefinition {
     }
 }
 
-#[derive(Debug, Clone, PartialEq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 pub struct GameEventEntry {
     pub name: String,
     pub kind: GameEventValueType,
 }
 
-#[derive(BitRead, BitWrite, Debug, Clone, Copy, PartialEq, Display)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(BitRead, BitWrite, Debug, Clone, Copy, PartialEq, Display, Serialize, Deserialize)]
 #[discriminant_bits = 3]
 pub enum GameEventValueType {
     None = 0,
@@ -54,7 +56,8 @@ pub enum GameEventValueType {
     Local = 7,
 }
 
-#[derive(Debug, Clone, PartialEq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 pub enum GameEventValue {
     String(String),
     Float(f32),
@@ -154,7 +157,8 @@ impl EventValue for () {
     }
 }
 
-#[derive(Debug, PartialEq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(Debug, PartialEq, Serialize, Deserialize)]
 pub struct RawGameEvent {
     pub event_type: GameEventType,
     pub values: Vec<GameEventValue>,
