@@ -1,6 +1,8 @@
 use bitbuffer::{BitRead, BitWrite};
+use serde::{Deserialize, Serialize};
 
-#[derive(BitRead, BitWrite, Debug, PartialEq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(BitRead, BitWrite, Debug, PartialEq, Serialize, Deserialize)]
 pub struct Header {
     #[size = 8]
     pub demo_type: String,
