@@ -12,6 +12,7 @@ use crate::demo::message::voice::*;
 use crate::demo::parser::{Encode, ParseBitSkip};
 use crate::{Parse, ParserState, Result, Stream};
 use bitbuffer::{BitRead, BitWrite, BitWriteStream, LittleEndian};
+use serde::{Deserialize, Serialize};
 use serde_repr::{Deserialize_repr, Serialize_repr};
 
 pub mod bspdecal;
@@ -25,6 +26,7 @@ pub mod tempentities;
 pub mod usermessage;
 pub mod voice;
 
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
 #[derive(
     BitRead, BitWrite, Debug, Clone, Copy, PartialEq, Eq, Serialize_repr, Deserialize_repr,
 )]
@@ -61,7 +63,10 @@ pub enum MessageType {
     CmdKeyValues = 32,
 }
 
-#[derive(Debug, PartialEq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(Debug, PartialEq, Serialize, Deserialize)]
+#[serde(bound(deserialize = "'a: 'static"))]
+#[serde(tag = "type")]
 pub enum Message<'a> {
     Empty,
     File(FileMessage),
