@@ -14,6 +14,7 @@ use std::fmt;
 use std::hint::unreachable_unchecked;
 use std::num::NonZeroU32;
 
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
 #[derive(
     Debug,
     Copy,
@@ -37,6 +38,7 @@ impl From<u32> for EntityId {
     }
 }
 
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
 #[derive(
     BitRead, BitWrite, Clone, Copy, Debug, PartialEq, Eq, Serialize_repr, Deserialize_repr,
 )]
@@ -49,7 +51,8 @@ pub enum PVS {
     Delete = 3,
 }
 
-#[derive(Debug, Clone, PartialEq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 pub struct PacketEntity {
     pub server_class: ClassId,
     pub entity_index: EntityId,
@@ -174,7 +177,8 @@ fn test_bit_var_roundtrip() {
     bit_var_normal(123456789);
 }
 
-#[derive(Debug, PartialEq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(Debug, PartialEq, Serialize, Deserialize)]
 pub struct PacketEntitiesMessage {
     pub entities: Vec<PacketEntity>,
     pub removed_entities: Vec<EntityId>,
@@ -378,13 +382,7 @@ impl PacketEntitiesMessage {
         state: &ParserState,
     ) -> Result<()> {
         let bits = log_base2(state.server_classes.len()) + 1;
-        let (class_index, _class) = state
-            .server_classes
-            .iter()
-            .enumerate()
-            .find(|(_, class)| entity.server_class == class.id)
-            .ok_or(ParseError::UnknownServerClass(entity.server_class))?;
-        class_index.write_sized(stream, bits as usize)?;
+        u16::from(entity.server_class).write_sized(stream, bits as usize)?;
         entity.serial_number.write_sized(stream, 10)?;
 
         Ok(())
