@@ -1,8 +1,10 @@
 use bitbuffer::{BitRead, BitReadStream, BitWrite, BitWriteStream, Endianness};
+use serde::{Deserialize, Serialize};
 
 use crate::ReadResult;
 
-#[derive(Debug, BitWrite, PartialEq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(Debug, BitWrite, PartialEq, Serialize, Deserialize)]
 pub struct ConVar {
     key: String,
     value: String,
@@ -20,7 +22,8 @@ impl<E: Endianness> BitRead<'_, E> for ConVar {
     }
 }
 
-#[derive(Debug, BitRead, PartialEq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(Debug, BitRead, PartialEq, Serialize, Deserialize)]
 pub struct SetConVarMessage {
     length: u8,
     #[size = "length"]
