@@ -5,7 +5,8 @@ use crate::demo::handle_utf8_error;
 
 use crate::{ReadResult, Stream};
 
-#[derive(BitRead, BitWrite, Clone, Copy, Debug, PartialEq, Eq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(BitRead, BitWrite, Clone, Copy, Debug, PartialEq, Eq, Serialize, Deserialize)]
 #[repr(u8)]
 #[discriminant_bits = 8]
 pub enum UserMessageType {
@@ -70,7 +71,10 @@ pub enum UserMessageType {
     Unknown = 255,
 }
 
-#[derive(Debug, PartialEq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(Debug, PartialEq, Serialize, Deserialize)]
+#[serde(bound(deserialize = "'a: 'static"))]
+#[serde(tag = "type")]
 pub enum UserMessage<'a> {
     SayText2(Box<SayText2Message>),
     Text(Box<TextMessage>),
@@ -78,7 +82,7 @@ pub enum UserMessage<'a> {
     Train(TrainMessage),
     VoiceSubtitle(VoiceSubtitleMessage),
     Shake(ShakeMessage),
-    Unknown(u8, UnknownUserMessage<'a>),
+    Unknown(UnknownUserMessage<'a>),
 }
 
 impl UserMessage<'_> {
@@ -90,7 +94,7 @@ impl UserMessage<'_> {
             UserMessage::Train(_) => UserMessageType::Train as u8,
             UserMessage::VoiceSubtitle(_) => UserMessageType::VoiceSubtitle as u8,
             UserMessage::Shake(_) => UserMessageType::Shake as u8,
-            UserMessage::Unknown(ty, _) => *ty,
+            UserMessage::Unknown(msg) => msg.raw_type,
         }
     }
 }
@@ -108,13 +112,19 @@ impl<'a> BitRead<'a, LittleEndian> for UserMessage<'a> {
                     UserMessageType::Train => UserMessage::Train(data.read()?),
                     UserMessageType::VoiceSubtitle => UserMessage::VoiceSubtitle(data.read()?),
                     UserMessageType::Shake => UserMessage::Shake(data.read()?),
-                    _ => UserMessage::Unknown(message_type as u8, data.read()?),
+                    _ => UserMessage::Unknown(UnknownUserMessage {
+                        raw_type: message_type as u8,
+                        data,
+                    }),
                 }
             }
             Err(BitError::UnmatchedDiscriminant { discriminant, .. }) => {
                 let length = stream.read_int(11)?;
-                let mut data = stream.read_bits(length)?;
-                UserMessage::Unknown(discriminant as u8, data.read()?)
+                let data = stream.read_bits(length)?;
+                UserMessage::Unknown(UnknownUserMessage {
+                    raw_type: discriminant as u8,
+                    data,
+                })
             }
             Err(e) => return Err(e),
         };
@@ -139,7 +149,7 @@ impl<'a> BitWrite<LittleEndian> for UserMessage<'a> {
             UserMessage::Train(body) => stream.write(body),
             UserMessage::VoiceSubtitle(body) => stream.write(body),
             UserMessage::Shake(body) => stream.write(body),
-            UserMessage::Unknown(_, body) => stream.write(body),
+            UserMessage::Unknown(body) => stream.write(&body.data),
         })?;
 
         Ok(())
@@ -158,6 +168,7 @@ fn test_user_message_roundtrip() {
     })));
 }
 
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
 #[derive(Debug, Clone, Copy, Serialize, Deserialize, PartialEq)]
 pub enum ChatMessageKind {
     #[serde(rename = "TF_Chat_All")]
@@ -204,7 +215,8 @@ impl BitWrite<LittleEndian> for ChatMessageKind {
     }
 }
 
-#[derive(Debug, Clone, PartialEq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 pub struct SayText2Message {
     pub client: u8,
     pub raw: u8,
@@ -312,7 +324,8 @@ fn test_say_text2_roundtrip() {
     });
 }
 
-#[derive(BitRead, BitWrite, Debug, Clone, PartialEq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(BitRead, BitWrite, Debug, Clone, PartialEq, Serialize, Deserialize)]
 #[discriminant_bits = 8]
 pub enum HudTextLocation {
     PrintNotify = 1,
@@ -321,31 +334,36 @@ pub enum HudTextLocation {
     PrintCenter,
 }
 
-#[derive(BitRead, BitWrite, Debug, Clone, PartialEq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(BitRead, BitWrite, Debug, Clone, PartialEq, Serialize, Deserialize)]
 pub struct TextMessage {
     pub location: HudTextLocation,
     pub text: String,
     pub substitute: [String; 4],
 }
 
-#[derive(BitRead, BitWrite, Debug, Clone, PartialEq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(BitRead, BitWrite, Debug, Clone, PartialEq, Serialize, Deserialize)]
 pub struct ResetHudMessage {
     pub data: u8,
 }
 
-#[derive(BitRead, BitWrite, Debug, Clone, PartialEq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(BitRead, BitWrite, Debug, Clone, PartialEq, Serialize, Deserialize)]
 pub struct TrainMessage {
     pub data: u8,
 }
 
-#[derive(BitRead, BitWrite, Debug, Clone, PartialEq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(BitRead, BitWrite, Debug, Clone, PartialEq, Serialize, Deserialize)]
 pub struct VoiceSubtitleMessage {
     client: u8,
     menu: u8,
     item: u8,
 }
 
-#[derive(BitRead, BitWrite, Debug, Clone, PartialEq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(BitRead, BitWrite, Debug, Clone, PartialEq, Serialize, Deserialize)]
 pub struct ShakeMessage {
     command: u8,
     amplitude: f32,
@@ -353,21 +371,10 @@ pub struct ShakeMessage {
     duration: f32,
 }
 
-#[derive(Debug, Clone, PartialEq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+#[serde(bound(deserialize = "'a: 'static"))]
 pub struct UnknownUserMessage<'a> {
+    raw_type: u8,
     data: Stream<'a>,
 }
-
-impl<'a> BitRead<'a, LittleEndian> for UnknownUserMessage<'a> {
-    fn read(stream: &mut Stream<'a>) -> ReadResult<Self> {
-        Ok(UnknownUserMessage {
-            data: stream.read_bits(stream.bits_left())?,
-        })
-    }
-}
-
-impl<'a> BitWrite<LittleEndian> for UnknownUserMessage<'a> {
-    fn write(&self, stream: &mut BitWriteStream<LittleEndian>) -> ReadResult<()> {
-        self.data.write(stream)
-    }
-}
