@@ -10,8 +10,22 @@ use std::cmp::min;
 use std::convert::TryFrom;
 use std::iter::once;
 
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
 #[derive(
-    BitRead, BitWrite, Debug, Clone, Copy, PartialEq, Eq, Hash, Ord, PartialOrd, Display, FromStr,
+    BitRead,
+    BitWrite,
+    Debug,
+    Clone,
+    Copy,
+    PartialEq,
+    Eq,
+    Hash,
+    Ord,
+    PartialOrd,
+    Display,
+    FromStr,
+    Serialize,
+    Deserialize,
 )]
 pub struct ClassId(u16);
 
@@ -33,6 +47,7 @@ impl From<ClassId> for u16 {
     }
 }
 
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
 #[derive(BitRead, BitWrite, PartialEq, Eq, Hash, Debug, Serialize, Deserialize, Clone, Display)]
 pub struct ServerClassName(String);
 
@@ -54,13 +69,15 @@ impl From<&str> for ServerClassName {
     }
 }
 
-#[derive(BitRead, BitWrite, Debug, Clone, PartialEq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(BitRead, BitWrite, Debug, Clone, PartialEq, Serialize, Deserialize)]
 pub struct ServerClass {
     pub id: ClassId,
     pub name: ServerClassName,
     pub data_table: SendTableName,
 }
 
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
 #[derive(
     BitRead,
     BitWrite,
@@ -96,7 +113,8 @@ impl From<&str> for SendTableName {
     }
 }
 
-#[derive(Debug, Clone, PartialEq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 pub struct ParseSendTable {
     pub name: SendTableName,
     pub props: Vec<RawSendPropDefinition>,
@@ -318,7 +336,8 @@ impl ParseSendTable {
     }
 }
 
-#[derive(Debug, Clone)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct SendTable {
     pub name: SendTableName,
     pub needs_decoder: bool,
@@ -326,7 +345,8 @@ pub struct SendTable {
     pub flattened_props: Vec<SendPropDefinition>,
 }
 
-#[derive(Debug, PartialEq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(Debug, PartialEq, Serialize, Deserialize)]
 pub struct DataTablePacket {
     pub tick: u32,
     pub tables: Vec<ParseSendTable>,
