@@ -10,6 +10,7 @@ use self::stringtable::StringTablePacket;
 use self::synctick::SyncTickPacket;
 use self::usercmd::UserCmdPacket;
 use crate::demo::parser::Encode;
+use serde::{Deserialize, Serialize};
 
 pub mod consolecmd;
 pub mod datatable;
@@ -19,7 +20,10 @@ pub mod stringtable;
 pub mod synctick;
 pub mod usercmd;
 
-#[derive(Debug, PartialEq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(Debug, PartialEq, Serialize, Deserialize)]
+#[serde(bound(deserialize = "'a: 'static"))]
+#[serde(tag = "type")]
 pub enum Packet<'a> {
     Sigon(MessagePacket<'a>),
     Message(MessagePacket<'a>),
@@ -31,6 +35,22 @@ pub enum Packet<'a> {
     StringTables(StringTablePacket<'a>),
 }
 
+impl Packet<'_> {
+    pub fn tick(&self) -> u32 {
+        match self {
+            Packet::Sigon(msg) => msg.tick,
+            Packet::Message(msg) => msg.tick,
+            Packet::SyncTick(msg) => msg.tick,
+            Packet::ConsoleCmd(msg) => msg.tick,
+            Packet::UserCmd(msg) => msg.tick,
+            Packet::DataTables(msg) => msg.tick,
+            Packet::Stop(msg) => msg.tick,
+            Packet::StringTables(msg) => msg.tick,
+        }
+    }
+}
+
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
 #[derive(BitRead, BitWrite, Debug, Clone, Copy, Eq, PartialEq)]
 #[discriminant_bits = 8]
 #[repr(u8)]
