@@ -1,10 +1,12 @@
 use bitbuffer::{BitRead, BitReadStream, BitWrite, BitWriteStream, LittleEndian};
+use serde::{Deserialize, Serialize};
 
-#[derive(Debug, PartialEq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(Debug, PartialEq, Serialize, Deserialize)]
 pub struct UserCmdPacket {
-    tick: u32,
-    sequence_out: u32,
-    cmd: UserCmd,
+    pub tick: u32,
+    pub sequence_out: u32,
+    pub cmd: UserCmd,
 }
 
 impl<'a> BitRead<'a, LittleEndian> for UserCmdPacket {
@@ -30,7 +32,8 @@ impl BitWrite<LittleEndian> for UserCmdPacket {
     }
 }
 
-#[derive(Debug, PartialEq, BitRead, BitWrite)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(Debug, PartialEq, BitRead, BitWrite, Serialize, Deserialize)]
 pub struct UserCmd {
     command_number: Option<u32>,
     tick_count: Option<u32>,
@@ -43,7 +46,8 @@ pub struct UserCmd {
     mouse_dy: Option<u16>,
 }
 
-#[derive(Debug, PartialEq, BitRead, BitWrite)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(Debug, PartialEq, BitRead, BitWrite, Serialize, Deserialize)]
 pub struct WeaponSelect {
     #[size = 11]
     select: u16,
