@@ -18,6 +18,7 @@ use std::fmt;
 use std::hash::Hash;
 use std::ops::BitOr;
 
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
 #[derive(
     BitRead,
     BitWrite,
@@ -58,7 +59,8 @@ impl From<&str> for SendPropName {
     }
 }
 
-#[derive(Debug, Clone)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct RawSendPropDefinition {
     pub prop_type: SendPropType,
     pub name: SendPropName,
@@ -252,7 +254,8 @@ impl BitWrite<LittleEndian> for RawSendPropDefinition {
     }
 }
 
-#[derive(BitRead, BitWrite, Copy, Clone, PartialEq, Debug, Display)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(BitRead, BitWrite, Copy, Clone, PartialEq, Debug, Display, Serialize, Deserialize)]
 #[discriminant_bits = 5]
 pub enum SendPropType {
     Int = 0,
@@ -310,9 +313,20 @@ pub enum SendPropFlag {
     NormalVarInt = 32,
 }
 
-#[derive(Debug, Copy, Clone, PartialEq, Default)]
+#[derive(Debug, Copy, Clone, PartialEq, Default, Serialize, Deserialize)]
 pub struct SendPropFlags(BitFlags<SendPropFlag>);
 
+#[cfg(feature = "schemars")]
+impl schemars::JsonSchema for SendPropFlags {
+    fn schema_name() -> String {
+        "SendPropFlags".into()
+    }
+
+    fn json_schema(gen: &mut schemars::gen::SchemaGenerator) -> schemars::schema::Schema {
+        u16::json_schema(gen)
+    }
+}
+
 impl BitOr<SendPropFlag> for SendPropFlags {
     type Output = SendPropFlags;
 
@@ -356,7 +370,8 @@ impl BitWrite<LittleEndian> for SendPropFlags {
     }
 }
 
-#[derive(Debug, Clone)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(Debug, Clone, Serialize, Deserialize)]
 pub enum FloatDefinition {
     Coord,
     CoordMP,
@@ -398,7 +413,8 @@ impl FloatDefinition {
     }
 }
 
-#[derive(Debug, Clone)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct SendPropDefinition {
     pub identifier: SendPropIdentifier,
     pub parse_definition: SendPropParseDefinition,
@@ -416,7 +432,8 @@ impl TryFrom<&RawSendPropDefinition> for SendPropDefinition {
     }
 }
 
-#[derive(Debug, Clone)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(Debug, Clone, Serialize, Deserialize)]
 pub enum SendPropParseDefinition {
     NormalVarInt {
         changes_often: bool,
@@ -543,6 +560,7 @@ impl TryFrom<&RawSendPropDefinition> for SendPropParseDefinition {
     }
 }
 
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
 #[derive(Debug, Clone, Serialize, Deserialize)]
 #[serde(untagged)]
 pub enum SendPropValue {
@@ -1068,7 +1086,10 @@ impl<'a> TryFrom<&'a SendPropValue> for &'a [SendPropValue] {
     }
 }
 
-#[derive(Debug, Clone, Copy, Ord, PartialOrd, Eq, PartialEq, Hash, Display)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(
+    Debug, Clone, Copy, Ord, PartialOrd, Eq, PartialEq, Hash, Display, Serialize, Deserialize,
+)]
 pub struct SendPropIdentifier(u64);
 
 impl SendPropIdentifier {
@@ -1084,7 +1105,8 @@ impl From<u64> for SendPropIdentifier {
     }
 }
 
-#[derive(Debug, Clone, Display, PartialEq)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[derive(Debug, Clone, Display, PartialEq, Serialize, Deserialize)]
 #[display("{index} = {value}")]
 pub struct SendProp {
     pub index: u32,
