@@ -61,7 +61,7 @@ impl EntityDump {
                 .props
                 .into_iter()
                 .map(|prop| {
-                    let (table_name, prop_name) = &prop_names[&prop.index];
+                    let (table_name, prop_name) = &prop_names[&prop.identifier];
                     (format!("{}.{}", table_name, prop_name), prop.value)
                 })
                 .collect(),
@@ -111,7 +111,7 @@ impl MessageHandler for EntityDumper {
             for prop_def in &table.props {
                 self.prop_names.insert(
                     prop_def.identifier(),
-                    (prop_def.owner_table.clone(), prop_def.name.clone()),
+                    (table.name.clone(), prop_def.name.clone()),
                 );
             }
         }
